"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var BluetoothLowEnergyService_1;
Object.defineProperty(exports, "__esModule", { value: true });
exports.BluetoothLowEnergyService = exports.NEW_DISTANCE_CHANNEL = void 0;
const common_1 = require("@nestjs/common");
const entities_service_1 = require("../../entities/entities.service");
const i_beacon_1 = require("./i-beacon");
const tag_1 = require("./tag");
const config_service_1 = require("../../config/config.service");
const cluster_service_1 = require("../../cluster/cluster.service");
const new_distance_event_1 = require("./new-distance.event");
const sensor_config_1 = require("../home-assistant/sensor-config");
const device_tracker_config_1 = require("../home-assistant/device-tracker-config");
const schedule_1 = require("@nestjs/schedule");
const filters_1 = require("../../util/filters");
const id_1 = require("../../util/id");
const home_assistant_const_1 = require("../home-assistant/home-assistant.const");
const _ = __importStar(require("lodash"));
const device_tracker_1 = require("../../entities/device-tracker");
const sensor_1 = require("../../entities/sensor");
const room_presence_proxy_1 = require("../../integration-support/room-presence/room-presence.proxy");
const bluetooth_low_energy_presence_sensor_1 = require("./bluetooth-low-energy-presence.sensor");
const bluetooth_service_1 = require("../../integration-support/bluetooth/bluetooth.service");
const promises_1 = require("../../util/promises");
const util = __importStar(require("util"));
const objects_1 = require("../../util/objects");
exports.NEW_DISTANCE_CHANNEL = 'bluetooth-low-energy.new-distance';
const APPLE_ADVERTISEMENT_ID = Buffer.from([0x4c, 0x00]);
let BluetoothLowEnergyService = BluetoothLowEnergyService_1 = class BluetoothLowEnergyService extends filters_1.KalmanFilterable(Object, 0.008, 4) {
    constructor(bluetoothService, entitiesService, configService, clusterService, schedulerRegistry) {
        super();
        this.bluetoothService = bluetoothService;
        this.entitiesService = entitiesService;
        this.configService = configService;
        this.clusterService = clusterService;
        this.schedulerRegistry = schedulerRegistry;
        this.seenIds = new Set();
        this.companionAppTags = new Map();
        this.companionAppDenylist = new Set();
        this.tagUpdaters = {};
        this.config = this.configService.get('bluetoothLowEnergy');
        this.logger = new common_1.Logger(BluetoothLowEnergyService_1.name);
    }
    onModuleInit() {
        if (!this.isAllowlistEnabled() && !this.isDenylistEnabled()) {
            this.logger.warn('The allowlist and denylist are empty, no sensors will be created! Please add some of the discovered IDs below to your configuration.');
        }
    }
    onApplicationBootstrap() {
        this.bluetoothService.onLowEnergyDiscovery(this.handleDiscovery.bind(this));
        this.clusterService.on(exports.NEW_DISTANCE_CHANNEL, this.handleNewDistance.bind(this));
        this.clusterService.subscribe(exports.NEW_DISTANCE_CHANNEL);
    }
    async handleDiscovery(peripheral) {
        let tag = this.createTag(peripheral);
        if (this.config.onlyIBeacon && !(tag instanceof i_beacon_1.IBeacon)) {
            return;
        }
        tag = await this.applyCompanionAppOverride(tag);
        if (!this.seenIds.has(tag.id)) {
            this.logger.log(`Discovered new BLE peripheral ${tag.name} with ID ${tag.id} and RSSI ${tag.rssi}`);
            this.seenIds.add(tag.id);
        }
        if ((this.isOnAllowlist(tag.id) ||
            (!this.isAllowlistEnabled() && this.isDenylistEnabled())) &&
            !this.isOnDenylist(tag.id)) {
            const appId = tag.isApp ? tag.id : undefined;
            tag = this.applyOverrides(tag);
            tag.rssi = tag.rssi * this.config.rssiFactor;
            tag.rssi = this.filterRssi(tag.id, tag.rssi);
            const globalSettings = this.configService.get('global');
            const event = new new_distance_event_1.NewDistanceEvent(globalSettings.instanceName, tag.id, tag.name, tag.peripheral.id, tag.rssi, tag.measuredPower, tag.distance, tag.distance > this.config.maxDistance, appId, tag instanceof i_beacon_1.IBeacon ? tag.batteryLevel : undefined);
            if (!this.tagUpdaters.hasOwnProperty(tag.id)) {
                this.tagUpdaters[tag.id] = _.throttle((event) => {
                    this.handleNewDistance(event);
                    this.clusterService.publish(exports.NEW_DISTANCE_CHANNEL, event);
                }, this.config.updateFrequency * 1000);
            }
            this.tagUpdaters[tag.id](event);
        }
    }
    async discoverCompanionAppId(tag) {
        const disconnectPromise = util
            .promisify(tag.peripheral.once)
            .bind(tag.peripheral)('disconnect');
        const peripheral = await this.bluetoothService.connectLowEnergyDevice(tag.peripheral);
        try {
            return await promises_1.promiseWithTimeout(Promise.race([
                BluetoothLowEnergyService_1.readCompanionAppId(peripheral),
                disconnectPromise,
            ]), 15 * 1000);
        }
        catch (e) {
            this.logger.error(`Failed to search for companion app at tag ${tag.id}: ${e.message}`, e.trace);
            if (e.message === 'timed out') {
                this.bluetoothService.resetHciDevice(this.bluetoothService.lowEnergyAdapterId);
            }
            return null;
        }
        finally {
            if (!['disconnecting', 'disconnected'].includes(tag.peripheral.state)) {
                this.bluetoothService.disconnectLowEnergyDevice(tag.peripheral);
            }
        }
    }
    handleNewDistance(event) {
        const sensorId = id_1.makeId(`ble ${event.tagId}`);
        let sensor;
        const hasBattery = event.batteryLevel !== undefined;
        if (event.isApp) {
            this.handleAppDiscovery(event.peripheralId, event.appId);
        }
        if (this.entitiesService.has(sensorId)) {
            sensor = this.entitiesService.get(sensorId);
        }
        else {
            sensor = this.createRoomPresenceSensor(sensorId, event.tagId, event.tagName, hasBattery);
        }
        sensor.handleNewMeasurement(event.instanceName, event.rssi, event.measuredPower, event.distance, event.outOfRange, event.batteryLevel);
    }
    isIBeacon(manufacturerData) {
        return (manufacturerData &&
            25 <= manufacturerData.length &&
            0x004c === manufacturerData.readUInt16LE(0) &&
            0x02 === manufacturerData.readUInt8(2) &&
            0x15 === manufacturerData.readUInt8(3));
    }
    isAllowlistEnabled() {
        var _a, _b;
        return (((_a = this.config.allowlist) === null || _a === void 0 ? void 0 : _a.length) > 0 || ((_b = this.config.whitelist) === null || _b === void 0 ? void 0 : _b.length) > 0);
    }
    isDenylistEnabled() {
        var _a, _b;
        return (((_a = this.config.denylist) === null || _a === void 0 ? void 0 : _a.length) > 0 || ((_b = this.config.blacklist) === null || _b === void 0 ? void 0 : _b.length) > 0);
    }
    isOnAllowlist(id) {
        const allowlist = [
            ...(this.config.allowlist || []),
            ...(this.config.whitelist || []),
        ];
        if (allowlist.length === 0) {
            return false;
        }
        return this.config.allowlistRegex || this.config.whitelistRegex
            ? allowlist.some((regex) => id.match(regex))
            : allowlist
                .map((x) => String(x).toLowerCase())
                .includes(id.toLowerCase());
    }
    isOnDenylist(id) {
        const denylist = [
            ...(this.config.denylist || []),
            ...(this.config.blacklist || []),
        ];
        if (denylist.length === 0) {
            return false;
        }
        return this.config.denylistRegex || this.config.blacklistRegex
            ? denylist.some((regex) => id.match(regex))
            : denylist.map((x) => String(x).toLowerCase()).includes(id.toLowerCase());
    }
    filterRssi(tagId, rssi) {
        return this.kalmanFilter(rssi, tagId);
    }
    createRoomPresenceSensor(sensorId, deviceId, deviceName, hasBattery) {
        const deviceInfo = {
            identifiers: deviceId,
            name: deviceName,
            viaDevice: home_assistant_const_1.DISTRIBUTED_DEVICE_ID,
        };
        const deviceTracker = this.createDeviceTracker(id_1.makeId(`${sensorId}-tracker`), `${deviceName} Tracker`, deviceInfo);
        let batterySensor;
        if (hasBattery) {
            batterySensor = this.createBatterySensor(id_1.makeId(`${sensorId}-battery`), `${deviceName} Battery`, deviceInfo);
        }
        const sensorName = `${deviceName} Room Presence`;
        const customizations = [
            {
                for: sensor_config_1.SensorConfig,
                overrides: {
                    icon: 'mdi:bluetooth',
                    device: deviceInfo,
                },
            },
        ];
        const rawSensor = new bluetooth_low_energy_presence_sensor_1.BluetoothLowEnergyPresenceSensor(sensorId, sensorName, this.config.timeout);
        const proxiedSensor = new Proxy(rawSensor, new room_presence_proxy_1.RoomPresenceProxyHandler(deviceTracker, batterySensor));
        const sensor = this.entitiesService.add(proxiedSensor, customizations);
        const interval = setInterval(sensor.updateState.bind(sensor), this.config.timeout * 1000);
        this.schedulerRegistry.addInterval(`${sensorId}_timeout_check`, interval);
        return sensor;
    }
    createDeviceTracker(id, name, deviceInfo) {
        const trackerCustomizations = [
            {
                for: device_tracker_config_1.DeviceTrackerConfig,
                overrides: {
                    device: deviceInfo,
                },
            },
        ];
        return this.entitiesService.add(new device_tracker_1.DeviceTracker(id, name, true), trackerCustomizations);
    }
    createBatterySensor(id, name, deviceInfo) {
        const batteryCustomizations = [
            {
                for: sensor_config_1.SensorConfig,
                overrides: {
                    deviceClass: 'battery',
                    unitOfMeasurement: '%',
                    device: deviceInfo,
                },
            },
        ];
        return this.entitiesService.add(new sensor_1.Sensor(id, name, true), batteryCustomizations);
    }
    createTag(peripheral) {
        if (this.config.processIBeacon &&
            this.isIBeacon(peripheral.advertisement.manufacturerData)) {
            return new i_beacon_1.IBeacon(peripheral, this.config.majorMask, this.config.minorMask, this.config.batteryMask);
        }
        else {
            return new tag_1.Tag(peripheral);
        }
    }
    applyOverrides(tag) {
        const overrides = objects_1.getPropertyCaseInsensitive(this.config.tagOverrides, tag.id);
        if (overrides != undefined) {
            if (overrides.id !== undefined) {
                tag.id = overrides.id;
            }
            if (overrides.name !== undefined) {
                tag.name = overrides.name;
            }
            if (overrides.measuredPower !== undefined) {
                tag.measuredPower = overrides.measuredPower;
            }
            if (overrides.batteryMask !== undefined && tag instanceof i_beacon_1.IBeacon) {
                tag.batteryMask = overrides.batteryMask;
            }
        }
        return tag;
    }
    async applyCompanionAppOverride(tag) {
        var _a, _b;
        const manufacturerData = (_b = (_a = tag.peripheral) === null || _a === void 0 ? void 0 : _a.advertisement) === null || _b === void 0 ? void 0 : _b.manufacturerData;
        if (!this.companionAppTags.has(tag.id) &&
            !this.companionAppDenylist.has(tag.id) &&
            tag.peripheral.connectable &&
            BluetoothLowEnergyService_1.isAppleDevice(manufacturerData) &&
            BluetoothLowEnergyService_1.overflowContainsCompanionApp(BluetoothLowEnergyService_1.extractOverflowArea(manufacturerData))) {
            let appId;
            this.logger.log(`Attempting app discovery for tag ${tag.id}`);
            this.logger.debug(`Tag ${tag.id} seems to broadcast the app with manufacturer data ${tag.peripheral.advertisement.manufacturerData.toString('hex')}`);
            try {
                appId = await this.discoverCompanionAppId(tag);
            }
            catch (e) {
                this.logger.warn(`Failed to discover companion app ID due to error: ${e.message}`);
            }
            if (appId != null) {
                this.logger.log(`Discovered companion app with ID ${appId} for tag ${tag.id}`);
                this.handleAppDiscovery(tag.id, appId);
            }
            else {
                this.logger.debug(`Tag ${tag.id} should have the companion app, retrying in 15s`);
                this.banDeviceFromDiscovery(tag.id, 15 * 1000);
            }
        }
        const appId = this.companionAppTags.get(tag.id);
        if (appId != null) {
            tag.id = appId;
            tag.isApp = true;
        }
        return tag;
    }
    handleAppDiscovery(tagId, appId) {
        const oldId = this.companionAppTags.get(tagId);
        if (!(oldId != null && appId == null)) {
            this.companionAppTags.set(tagId, appId);
            this.companionAppDenylist.delete(tagId);
        }
    }
    banDeviceFromDiscovery(tagId, duration) {
        this.companionAppDenylist.add(tagId);
        setTimeout(() => this.companionAppDenylist.delete(tagId), duration);
    }
    static isAppleDevice(manufacturerData) {
        return manufacturerData === null || manufacturerData === void 0 ? void 0 : manufacturerData.slice(0, 2).equals(APPLE_ADVERTISEMENT_ID);
    }
    static extractOverflowArea(manufacturerData) {
        if (manufacturerData == null ||
            manufacturerData.length < 19 ||
            manufacturerData.readUInt8(manufacturerData.length - 17) != 0x01) {
            return null;
        }
        return manufacturerData.slice(manufacturerData.length - 16);
    }
    static overflowContainsCompanionApp(overflowArea) {
        return overflowArea != null && ((overflowArea.readUInt8(3) >> 4) & 1) === 1;
    }
    static async readCharacteristic(peripheral, serviceUuid, characteristicUuid) {
        const services = await peripheral.discoverServicesAsync([serviceUuid]);
        if (services.length > 0) {
            const characteristics = await services[0].discoverCharacteristicsAsync([
                characteristicUuid,
            ]);
            if (characteristics.length > 0) {
                const data = await characteristics[0].readAsync();
                return data.toString('utf-8');
            }
        }
        return null;
    }
    static readCompanionAppId(peripheral) {
        return this.readCharacteristic(peripheral, '5403c8a75c9647e99ab859e373d875a7', '21c46f33e813440786012ad281030052');
    }
};
BluetoothLowEnergyService = BluetoothLowEnergyService_1 = __decorate([
    common_1.Injectable(),
    __metadata("design:paramtypes", [bluetooth_service_1.BluetoothService,
        entities_service_1.EntitiesService,
        config_service_1.ConfigService,
        cluster_service_1.ClusterService,
        schedule_1.SchedulerRegistry])
], BluetoothLowEnergyService);
exports.BluetoothLowEnergyService = BluetoothLowEnergyService;
//# sourceMappingURL=bluetooth-low-energy.service.js.map