/// <reference types="node" />
export declare const SERVICE_DATA_UUID = "fe95";
declare class FrameControl {
    isFactoryNew: boolean;
    isConnected: boolean;
    isCentral: boolean;
    isEncrypted: boolean;
    hasMacAddress: boolean;
    hasCapabilities: boolean;
    hasEvent: boolean;
    hasCustomData: boolean;
    hasSubtitle: boolean;
    hasBinding: boolean;
}
declare class Capabilities {
    connectable: boolean;
    central: boolean;
    secure: boolean;
    io: boolean;
}
export declare const EventTypes: {
    temperature: number;
    humidity: number;
    illuminance: number;
    moisture: number;
    fertility: number;
    battery: number;
    temperatureAndHumidity: number;
};
export declare class Event {
    temperature?: number;
    humidity?: number;
    illuminance?: number;
    moisture?: number;
    fertility?: number;
    battery?: number;
}
export declare class ServiceData {
    frameControl: FrameControl;
    version: number;
    productId: number;
    productName: string;
    frameCounter: number;
    macAddress: string;
    capabilities: Capabilities;
    eventType?: number;
    eventLength?: number;
    event: Event;
}
export declare class Parser {
    private buffer;
    private readonly bindKey?;
    private baseByteLength;
    private frameControl;
    private eventType;
    constructor(buffer: Buffer, bindKey?: string | null);
    parse(): ServiceData;
    parseFrameControl(): FrameControl;
    parseVersion(): number;
    parseProductId(): number;
    parseProductName(): string;
    parseFrameCounter(): number;
    parseMacAddress(): string;
    get capabilityOffset(): number;
    parseCapabilities(): Capabilities;
    get eventOffset(): number;
    parseEventType(): number | null;
    parseEventLength(): number | null;
    decryptPayload(): void;
    parseEventData(): Event | null;
    parseTemperatureEvent(): Event;
    parseHumidityEvent(): Event;
    parseBatteryEvent(): Event;
    parseTemperatureAndHumidityEvent(): Event;
    parseIlluminanceEvent(): Event;
    parseFertilityEvent(): Event;
    parseMoistureEvent(): Event;
    toString(): string;
}
export {};
