"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Parser = exports.ServiceData = exports.Event = exports.EventTypes = exports.SERVICE_DATA_UUID = void 0;
const crypto = __importStar(require("crypto"));
exports.SERVICE_DATA_UUID = 'fe95';
const ProductNames = {
    152: 'Mi Flora HHCCJCY01',
    1371: 'Mijia LYWSD03MMC',
    1115: 'Mijia LYWSD02',
    426: 'Miija LYWSDCGQ',
};
const FrameControlFlags = {
    isFactoryNew: 1 << 0,
    isConnected: 1 << 1,
    isCentral: 1 << 2,
    isEncrypted: 1 << 3,
    hasMacAddress: 1 << 4,
    hasCapabilities: 1 << 5,
    hasEvent: 1 << 6,
    hasCustomData: 1 << 7,
    hasSubtitle: 1 << 8,
    hasBinding: 1 << 9,
};
class FrameControl {
}
const CapabilityFlags = {
    connectable: 1 << 0,
    central: 1 << 1,
    secure: 1 << 2,
    io: (1 << 3) | (1 << 4),
};
class Capabilities {
}
exports.EventTypes = {
    temperature: 4100,
    humidity: 4102,
    illuminance: 4103,
    moisture: 4104,
    fertility: 4105,
    battery: 4106,
    temperatureAndHumidity: 4109,
};
class Event {
}
exports.Event = Event;
class ServiceData {
}
exports.ServiceData = ServiceData;
class Parser {
    constructor(buffer, bindKey = null) {
        this.baseByteLength = 5;
        if (buffer == null) {
            throw new Error('A buffer must be provided.');
        }
        this.buffer = buffer;
        if (buffer.length < this.baseByteLength) {
            throw new Error(`Service data length must be >= 5 bytes. ${this.toString()}`);
        }
        this.bindKey = bindKey;
    }
    parse() {
        this.frameControl = this.parseFrameControl();
        const version = this.parseVersion();
        const productId = this.parseProductId();
        const productName = this.parseProductName();
        const frameCounter = this.parseFrameCounter();
        const macAddress = this.parseMacAddress();
        const capabilities = this.parseCapabilities();
        if (this.frameControl.isEncrypted) {
            this.decryptPayload();
        }
        this.eventType = this.parseEventType();
        const eventLength = this.parseEventLength();
        const event = this.parseEventData();
        return {
            frameControl: this.frameControl,
            version,
            productId,
            productName,
            frameCounter,
            macAddress,
            capabilities,
            eventType: this.eventType,
            eventLength,
            event,
        };
    }
    parseFrameControl() {
        const frameControl = this.buffer.readUInt16LE(0);
        return Object.keys(FrameControlFlags).reduce((map, flag) => {
            map[flag] = (frameControl & FrameControlFlags[flag]) !== 0;
            return map;
        }, new FrameControl());
    }
    parseVersion() {
        return this.buffer.readUInt8(1) >> 4;
    }
    parseProductId() {
        return this.buffer.readUInt16LE(2);
    }
    parseProductName() {
        const id = this.parseProductId();
        return id in ProductNames ? ProductNames[id] : 'Unknown';
    }
    parseFrameCounter() {
        return this.buffer.readUInt8(4);
    }
    parseMacAddress() {
        if (!this.frameControl.hasMacAddress) {
            return null;
        }
        const macBuffer = this.buffer.slice(this.baseByteLength, this.baseByteLength + 6);
        return Buffer.from(macBuffer).reverse().toString('hex');
    }
    get capabilityOffset() {
        if (!this.frameControl.hasMacAddress) {
            return this.baseByteLength;
        }
        return 11;
    }
    parseCapabilities() {
        if (!this.frameControl.hasCapabilities) {
            return null;
        }
        const capabilities = this.buffer.readUInt8(this.capabilityOffset);
        return Object.keys(CapabilityFlags).reduce((map, flag) => {
            map[flag] = (capabilities & CapabilityFlags[flag]) !== 0;
            return map;
        }, new Capabilities());
    }
    get eventOffset() {
        let offset = this.baseByteLength;
        if (this.frameControl.hasMacAddress) {
            offset = 11;
        }
        if (this.frameControl.hasCapabilities) {
            offset += 1;
        }
        return offset;
    }
    parseEventType() {
        if (!this.frameControl.hasEvent) {
            return null;
        }
        return this.buffer.readUInt16LE(this.eventOffset);
    }
    parseEventLength() {
        if (!this.frameControl.hasEvent) {
            return null;
        }
        return this.buffer.readUInt8(this.eventOffset + 2);
    }
    decryptPayload() {
        const msgLength = this.buffer.length;
        const eventLength = msgLength - this.eventOffset;
        if (eventLength < 3) {
            return;
        }
        if (this.bindKey == null) {
            throw Error('Sensor data is encrypted. Please configure a bindKey.');
        }
        const encryptedPayload = this.buffer.slice(this.eventOffset, msgLength);
        const nonce = Buffer.concat([
            this.buffer.slice(5, 11),
            this.buffer.slice(2, 4),
            this.buffer.slice(4, 5),
            encryptedPayload.slice(-7, -4),
        ]);
        const decipher = crypto.createDecipheriv('aes-128-ccm', Buffer.from(this.bindKey, 'hex'), nonce, { authTagLength: 4 });
        const ciphertext = encryptedPayload.slice(0, -7);
        decipher.setAuthTag(encryptedPayload.slice(-4));
        decipher.setAAD(Buffer.from('11', 'hex'), {
            plaintextLength: ciphertext.length,
        });
        const receivedPlaintext = decipher.update(ciphertext);
        decipher.final();
        this.buffer = Buffer.concat([
            this.buffer.slice(0, this.eventOffset),
            receivedPlaintext,
        ]);
    }
    parseEventData() {
        if (!this.frameControl.hasEvent) {
            return null;
        }
        switch (this.eventType) {
            case exports.EventTypes.temperature: {
                return this.parseTemperatureEvent();
            }
            case exports.EventTypes.humidity: {
                return this.parseHumidityEvent();
            }
            case exports.EventTypes.battery: {
                return this.parseBatteryEvent();
            }
            case exports.EventTypes.temperatureAndHumidity: {
                return this.parseTemperatureAndHumidityEvent();
            }
            case exports.EventTypes.illuminance: {
                return this.parseIlluminanceEvent();
            }
            case exports.EventTypes.fertility: {
                return this.parseFertilityEvent();
            }
            case exports.EventTypes.moisture: {
                return this.parseMoistureEvent();
            }
            default: {
                throw new Error(`Unknown event type: ${this.eventType}. ${this.toString()}`);
            }
        }
    }
    parseTemperatureEvent() {
        return {
            temperature: this.buffer.readInt16LE(this.eventOffset + 3) / 10,
        };
    }
    parseHumidityEvent() {
        return {
            humidity: this.buffer.readUInt16LE(this.eventOffset + 3) / 10,
        };
    }
    parseBatteryEvent() {
        return {
            battery: this.buffer.readUInt8(this.eventOffset + 3),
        };
    }
    parseTemperatureAndHumidityEvent() {
        const temperature = this.buffer.readInt16LE(this.eventOffset + 3) / 10;
        const humidity = this.buffer.readUInt16LE(this.eventOffset + 5) / 10;
        return { temperature, humidity };
    }
    parseIlluminanceEvent() {
        return {
            illuminance: this.buffer.readUIntLE(this.eventOffset + 3, 3),
        };
    }
    parseFertilityEvent() {
        return {
            fertility: this.buffer.readInt16LE(this.eventOffset + 3),
        };
    }
    parseMoistureEvent() {
        return {
            moisture: this.buffer.readInt8(this.eventOffset + 3),
        };
    }
    toString() {
        return this.buffer.toString('hex');
    }
}
exports.Parser = Parser;
//# sourceMappingURL=parser.js.map