"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var XiaomiMiService_1;
Object.defineProperty(exports, "__esModule", { value: true });
exports.XiaomiMiService = void 0;
const common_1 = require("@nestjs/common");
const entities_service_1 = require("../../entities/entities.service");
const config_service_1 = require("../../config/config.service");
const home_assistant_const_1 = require("../home-assistant/home-assistant.const");
const id_1 = require("../../util/id");
const parser_1 = require("./parser");
const sensor_1 = require("../../entities/sensor");
const sensor_config_1 = require("../home-assistant/sensor-config");
const bluetooth_service_1 = require("../../integration-support/bluetooth/bluetooth.service");
let XiaomiMiService = XiaomiMiService_1 = class XiaomiMiService {
    constructor(bluetoothService, entitiesService, configService) {
        this.bluetoothService = bluetoothService;
        this.entitiesService = entitiesService;
        this.configService = configService;
        this.config = {};
        this.logger = new common_1.Logger(XiaomiMiService_1.name);
    }
    onModuleInit() {
        this.config = {};
        this.configService.get('xiaomiMi').sensors.forEach((options) => {
            this.config[options.address] = options;
        });
        if (!this.hasSensors()) {
            this.logger.warn('No sensors entries in the config, so no sensors will be created! ' +
                'Enable the bluetooth-low-energy integration to log discovered IDs.');
        }
    }
    onApplicationBootstrap() {
        this.bluetoothService.onLowEnergyDiscovery(this.handleDiscovery.bind(this));
    }
    hasSensors() {
        return Object.keys(this.config).length > 0;
    }
    recordMeasure(device, kind, devClass, units, state) {
        this.logger.debug(`${device.name}: ${kind}: ${state}${units}`);
        const id = id_1.makeId(`xiaomi ${device.identifiers} ${kind}`);
        let entity = this.entitiesService.get(id);
        if (!entity) {
            const customizations = [
                {
                    for: sensor_config_1.SensorConfig,
                    overrides: {
                        deviceClass: devClass,
                        unitOfMeasurement: units,
                        device: device,
                    },
                },
            ];
            entity = this.entitiesService.add(new sensor_1.Sensor(id, `${device.name} ${kind}`, true, false), customizations);
        }
        entity.state = state;
    }
    handleDiscovery(peripheral) {
        const { advertisement, id } = peripheral || {};
        const options = this.config[id];
        if (!options) {
            return;
        }
        const buffer = XiaomiMiService_1.getValidServiceData(advertisement);
        if (!buffer) {
            this.logger.warn(`${options.name} does not appear to be a Xiaomi device. Got advertisement ${JSON.stringify(advertisement)}`);
            return;
        }
        let serviceData = null;
        try {
            serviceData = XiaomiMiService_1.parseServiceData(buffer, options.bindKey);
        }
        catch (error) {
            this.logger.error(`${options.name}: couldn't parse service data: ${error}`);
            return;
        }
        if (!serviceData.frameControl.hasEvent) {
            this.logger.debug(`${options.name}: advertisement with no event: ${buffer.toString('hex')}`);
            return;
        }
        const device = {
            name: options.name,
            manufacturer: 'Xiaomi',
            model: serviceData.productName,
            swVersion: serviceData.version.toString(),
            identifiers: peripheral.id,
            viaDevice: home_assistant_const_1.DISTRIBUTED_DEVICE_ID,
        };
        const event = serviceData.event;
        switch (serviceData.eventType) {
            case parser_1.EventTypes.temperature: {
                this.recordMeasure(device, 'Temperature', 'temperature', '°C', event.temperature);
                break;
            }
            case parser_1.EventTypes.humidity: {
                this.recordMeasure(device, 'Humidity', 'humidity', '%', event.humidity);
                break;
            }
            case parser_1.EventTypes.battery: {
                this.recordMeasure(device, 'Battery', 'battery', '%', event.battery);
                break;
            }
            case parser_1.EventTypes.temperatureAndHumidity: {
                this.recordMeasure(device, 'Temperature', 'temperature', '°C', event.temperature);
                this.recordMeasure(device, 'Humidity', 'humidity', '%', event.humidity);
                break;
            }
            case parser_1.EventTypes.illuminance: {
                this.recordMeasure(device, 'Illuminance', 'illuminance', 'lx', event.illuminance);
                break;
            }
            case parser_1.EventTypes.moisture: {
                this.recordMeasure(device, 'Moisture', undefined, '%', event.moisture);
                break;
            }
            case parser_1.EventTypes.fertility: {
                this.recordMeasure(device, 'Conductivity', undefined, 'µS/cm', event.fertility);
                break;
            }
            default: {
                this.logger.error(`${options.name}: unknown event type: ${serviceData.eventType}, ` +
                    `raw data: ${buffer.toString('hex')}`);
                break;
            }
        }
    }
    static getValidServiceData(advertisement) {
        if (!advertisement || !advertisement.serviceData) {
            return null;
        }
        const uuidPair = advertisement.serviceData.find((data) => data.uuid.toLowerCase() === parser_1.SERVICE_DATA_UUID);
        if (!uuidPair) {
            return null;
        }
        return uuidPair.data;
    }
    static parseServiceData(buffer, bindKey) {
        return new parser_1.Parser(buffer, bindKey).parse();
    }
};
XiaomiMiService = XiaomiMiService_1 = __decorate([
    common_1.Injectable(),
    __metadata("design:paramtypes", [bluetooth_service_1.BluetoothService,
        entities_service_1.EntitiesService,
        config_service_1.ConfigService])
], XiaomiMiService);
exports.XiaomiMiService = XiaomiMiService;
//# sourceMappingURL=xiaomi-mi.service.js.map